package com.yidian.common.services

import android.app.Activity
import android.util.Base64
import com.google.gson.Gson
import com.luck.picture.lib.PictureSelector
import com.luck.picture.lib.config.PictureMimeType
import com.luck.picture.lib.entity.LocalMedia
import com.luck.picture.lib.listener.OnResultCallbackListener
import com.yidian.common.bean.SelectImageParamsBean
import com.yidian.common.bean.SelectedImageBean
import com.yidian.common.bean.UploadImageBean
import com.yidian.common.utils.GlideEngine
import com.yidian.common.utils.KS3Core
import com.yidian.xarc.xrouter.ResultResolver
import com.yidian.xpage.observer.DStackActivityManager
import com.yidian.yac.core.zap.ZapService
import org.json.JSONArray
import org.json.JSONObject
import java.io.File
import java.io.FileInputStream

/**
 * author: yinjiacheng
 * date: 7/7/21 7:38 PM
 * description: 图片服务
 * 提供调用原生相机拍照、图片选择器、图片预览、图片上传能力
 */
// TODO: 7/9/21 动态获取权限
// TODO: 7/9/21 清除缓存
class ImageService : ZapService() {

  companion object {
    private const val TAG = "ImageService"
    const val SERVICE_NAME = "imageService"

    // 选择图片
    const val ACTION_SELECT_PIC = "selectImage"

    // 拍照
    const val ACTION_TAKE_PIC = "takePhoto"

    // 上传图片
    const val ACTION_UPLOAD_PIC = "uploadImage"

    // 预览图片 本期不做
    const val ACTION_PREVIEW_PIC = "previewPic"

    // 权限获取失败
    private const val CODE_PERMISSION_GRANT_FAIL = 20301

    // 选择失败、选择取消、拍照取消
    private const val CODE_ACTION_CANCEL = 20302

    // 文件不存在
    private const val CODE_FILE_NOT_EXIST = 20306

    // 文件上传失败
    private const val CODE_FILE_UPLOAD_FAIL = 20307

    // 图片最大选择数量
    private const val SELECT_PIC_MAX_COUNT = 9
  }

  private val gson by lazy { Gson() }
  private lateinit var resolver: ResultResolver

  /**
   * 当前栈顶Activity实例
   */
  private lateinit var curTopActivity: Activity

  /**
   * 内存缓存已选择图片
   */
  private val cacheSelectedImages by lazy { ArrayList<LocalMedia>() }

  override fun onAction(path: String, action: String, params: Any, options: Any, resolver: ResultResolver) {
    this.resolver = resolver
    curTopActivity = DStackActivityManager.getInstance().topActivity
    val resolveParams = params as JSONObject
    when (action) {
      ACTION_SELECT_PIC -> {
        selectPic(gson.fromJson(resolveParams.toString(), SelectImageParamsBean::class.java))
      }
      ACTION_TAKE_PIC -> {
        takePic()
      }
      ACTION_UPLOAD_PIC -> {
        uploadPic(resolveParams.optString("id"))
      }
      ACTION_PREVIEW_PIC -> {
        // 本期不做
      }
    }
  }

  override fun onExit() {

  }

  override fun onInit() {

  }

  /**
   * 选择图片
   */
  private fun selectPic(params: SelectImageParamsBean) {
    PictureSelector.create(curTopActivity)
      .openGallery(PictureMimeType.ofImage())
      .imageEngine(GlideEngine.createGlideEngine())
      .maxSelectNum(params.maxCount)
      .isWeChatStyle(true)
      .isCompress(true)
      .compressQuality(10)
      .forResult(object : OnResultCallbackListener<LocalMedia> {
        override fun onResult(result: MutableList<LocalMedia>?) {
          if (result.isNullOrEmpty()) {
            resolver.fail(CODE_ACTION_CANCEL, "select failed", "选择失败")
            return
          }
          cacheSelectedImages.addAll(result as ArrayList<LocalMedia>)
          val picList = ArrayList<SelectedImageBean>(params.maxCount)
          result.forEach {
            picList.add(
              SelectedImageBean(
                result[0].id.toString(),
                Base64.encodeToString(FileInputStream(it.compressPath).readBytes(), Base64.DEFAULT)
              )
            )
          }
          // 成功回传数据
          resolver.success(JSONArray(gson.toJson(picList)))
        }

        override fun onCancel() {
          resolver.fail(CODE_ACTION_CANCEL, "select cancel", "选择取消")
        }

      })
//    ToastUtils.showShort("请到设置中授予存储空间权限")
//    resolver.fail(CODE_PERMISSION_GRANT_FAIL, "check permission READ_EXTERNAL_STORAGE failed", "获取读取外部存储权限失败")
  }

  /**
   * 拍摄图片
   */
  private fun takePic() {
    PictureSelector.create(curTopActivity)
      .openCamera(PictureMimeType.ofImage())
      .imageEngine(GlideEngine.createGlideEngine())
      .isCompress(true)
      .compressQuality(10)
      .forResult(object : OnResultCallbackListener<LocalMedia> {
        override fun onResult(result: MutableList<LocalMedia>?) {
          if (result.isNullOrEmpty()) {
            resolver.fail(CODE_ACTION_CANCEL, "take failed", "拍照失败")
            return
          }
          cacheSelectedImages.add(result[0])
          // 成功回传数据
          resolver.success(
            JSONArray(
              gson.toJson(
                arrayListOf(
                  SelectedImageBean(
                    result[0].id.toString(),
                    Base64.encodeToString(FileInputStream(result[0].compressPath).readBytes(), Base64.DEFAULT)
                  )
                )
              )
            )
          )
        }

        override fun onCancel() {
          resolver.fail(CODE_ACTION_CANCEL, "take cancel", "拍照取消")
        }
      })
//    ToastUtils.showShort("请到设置中授予相机权限")
//    resolver.fail(CODE_PERMISSION_GRANT_FAIL, "check permission CAMERA failed", "获取摄像头权限失败")
  }

  /**
   * 上传图片
   * @param id 需要上传的图片id
   */
  private fun uploadPic(id: String) {
    var uploadPic: LocalMedia? = null
    cacheSelectedImages.forEach {
      // 遍历寻找指定图片
      if (id == it.id.toString()) {
        uploadPic = it
        return@forEach
      }
    }
    uploadPic?.let {
      val file = File(it.realPath)
      if (!file.exists()) {
        resolver.fail(CODE_FILE_NOT_EXIST, "file not exist", "文件不存在")
        return
      }
      KS3Core.INSTANCE.uploadObject(file, KS3Core.ObjectType.IMAGE, KS3Core.ScenarioType.COMMODITY, object : KS3Core.OnKS3TaskListener {
        override fun onTaskStart() {

        }

        override fun onTaskProgress(progress: Double) {

        }

        override fun onTaskFinish() {

        }

        override fun onTaskCancel() {

        }

        override fun onTaskSuccess(bucket: String?, objectKey: String?) {
          resolver.success(JSONObject(gson.toJson(UploadImageBean(id, "${bucket}/${objectKey}"))))
        }

        override fun onTaskFailure(statesCode: Int, message: String?) {
          resolver.fail(CODE_FILE_UPLOAD_FAIL, "upload failed", "上传失败")
        }
      })
    }
  }
}
